using System;
using System.Collections.Generic;
using System.Text;
//using Visma.Core.Deployment.Target.Validator;
//using Visma.Core.Deployment.Target.IPC;
using System.Windows.Forms;
//using Visma.Core.Deployment.Target.Validator.Extensions;
using System.Net;
using Visma.Services.OPGateway.Shared;
using Visma.Services.MGOPGCommon.Shared;
using System.Net.Sockets;
using InstallerShared;
using Visma.Services.PlatformValidator.Core;
using Visma.Services.PlatformValidator.Core.Extensions;
using Visma.Services.OPGateway.FirewallUtility;
using Common;

public class ConnectODG : IssueExtension
{
    string hostname = "odg.visma.net";
    int portno = 0;
    private IssueExtensionArgument m_arg;

    public override IssueState Detect(IssueExtensionArgument arg)
    {

        try
        {
            m_arg = arg;
            string path = Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData) + @"\Visma\Visma Installer\sshport.txt";
            string addr = FilesUtil.ReadStreamFile(path);
            string[] addrsplit = addr.Split(new[] {':'}, StringSplitOptions.RemoveEmptyEntries);
            if (addrsplit.Length > 1)
            {
            hostname = addrsplit[0];
            portno = Convert.ToInt32(addrsplit[1]);
            }
            else
            {
            portno = Convert.ToInt32(addrsplit[0]);
            }
           
            if (!CanConnect())
            {
                if (m_arg.DoRepair)
                {
                    // Platform Validator wants us to repair - try to do so.
                    bool succeeded1 = FirewallUtil.AllowOutgoingConnection(portno);
                    if (succeeded1)
                        m_arg.Reporter.Log(string.Format(GetText("OpenPortOK"), portno) + Environment.NewLine);
                    else
                        m_arg.Reporter.Warn(string.Format(GetText("OpenPortNOK"), portno) + Environment.NewLine);
                    return CanConnect() ? IssueState.Success : IssueState.Failed;
                }
                else
                {
                    return IssueState.Failed;
                }
            }
            else
            {
                return IssueState.Success;
            }

        }
        catch (Exception ex)
        {
            arg.Reporter.Log(GetText("ODGConnectionNOK") + hostname + ":" + portno.ToString());
            arg.Reporter.Log(ex.ToString());
            return IssueState.Failed;
        }

        //if ( arg.AllowGui )
        //{
        //   // We're allowed to show a GUI - if you do, use arg.ParentWindow as parent window
        //}

        //if ( arg.DoRepair )
        //{
        //   // Platform Validator wants us to repair - try to do so.
        //}

        //arg.Reporter.Log( "Hello world from Template" );
        //arg.Reporter.Warn(GetText("DemoText"));			// for localization - DemoText refers to text in Xml-file



        //// Return one of the following
        //return IssueState.NotRelevant;	// This issue is not relevant - Will not be shown in issue listy
        //return IssueState.Unresolved;	// Unable to evaluate whether the issue is a problem or not
        //return IssueState.Failed;		// The issue is a problem -  icon in list
        //return IssueState.Success;	// The issue is not a problem - green icon in list
    }

    private bool CanConnect()
    {
        bool canConnect = false;
        try
        {
            ILibLogger logger = new OpgInstallerLogger();
            ConnectionSettings proxySettings = IssuesHelper.GetConnectionSettings();
            canConnect = new TcpClientWithTimeout(hostname, portno, 15000, logger, proxySettings).Connect();
            if (canConnect)
                m_arg.Reporter.Log(GetText("ODGConnectionOK") + hostname + ":" + portno.ToString());
            else           
                m_arg.Reporter.Log(GetText("ODGConnectionNOK") + hostname + ":" + portno.ToString() + Environment.NewLine + "See logs!");            
        }
        catch (System.Net.Sockets.SocketException ex)
        {
            m_arg.Reporter.Log(GetText("ODGConnectionNOK") + hostname + ":" + portno.ToString() + Environment.NewLine);
            m_arg.Reporter.Log("SocketException: " + ex.ErrorCode + " " + ex.Message + Environment.NewLine);
            m_arg.Reporter.Log(string.Format(GetText("InformFirewallRule"), portno.ToString()) + Environment.NewLine);
        }

        return canConnect;
    }
}
